
!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 10/20/2010 13:55:45.797'!
currentIndexChanged: aModelIndex

	self emitAlways.! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/20/2010 14:08:00.82'!
currentModelIndex

	^ self viewport currentModelIndex! !

!UiAbstractItemView methodsFor: 'initialization' stamp: 'mt 10/20/2010 13:56:11.232'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: self viewportClass new.
	self connect: self viewport signal: #focusChanged: toSelector: #scrollToShow:.
	self connect: self viewport signal: #clicked: toSelector: #clicked:.
	self connect: self viewport signal: #doubleClicked: toSelector: #doubleClicked:.
	self connect: self viewport signal: #selectionChanged: toSelector: #selectionChanged:.
	self connect: self viewport signal: #currentIndexChanged: toSelector: #currentIndexChanged:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	self defaultItemDelegate: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/20/2010 14:07:37.965'!
selection

	^ self viewport selectedModelIndices! !

!UiAbstractItemView methodsFor: 'signals' stamp: 'mt 10/20/2010 13:42:53.216'!
selectionChanged: someModelIndices

	self emitAlways.! !

UiAbstractItemView subclass: #UiListIconView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListIconView commentStamp: 'mt 10/20/2010 14:02:59.798' prior: 0!
I am a list view that is able to break the line of items in one direction and shows as many items as possible.

I map some signals to ones that only indicate row indices, e.g., selected row indices instead of whole model indices.!

!UiListIconView methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:45:12.941'!
currentRow

	^ self currentModelIndex ifNotNil: [:cmi | cmi row]! !

!UiListIconView methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:10:14.822'!
currentRow: anIndex

	anIndex ifNil: [^ self viewport selection currentCell: nil].
	self viewport selection currentCell: (1@anIndex).! !

!UiListIconView methodsFor: 'signals' stamp: 'mt 10/20/2010 13:53:08.525'!
currentRowChanged: newRowIndex

	self emit.! !

!UiListIconView methodsFor: 'initialization' stamp: 'mt 10/20/2010 14:02:19.337'!
initialize

	super initialize.
	
	self connect: self viewport signal: #currentIndexChanged: toSelector: #onCurrentIndexChanged:.
	self connect: self viewport signal: #selectionChanged: toSelector: #onSelectionChanged:.! !

!UiListIconView methodsFor: 'updating' stamp: 'mt 10/20/2010 14:44:47.163'!
onCurrentIndexChanged: aModelIndex
	"Map to the row index. Ignore the column in a list."
	
	self currentRowChanged: (aModelIndex ifNotNil: [:m | m row]).! !

!UiListIconView methodsFor: 'updating' stamp: 'mt 10/20/2010 14:01:11.915'!
onSelectionChanged: someModelIndices

	self selectedRowsChanged: (someModelIndices collect: [:i | i row]).! !

!UiListIconView methodsFor: 'signals' stamp: 'mt 10/20/2010 14:00:30.024'!
selectedRowsChanged: someRowIndices

	self emit.! !

!UiListIconView methodsFor: 'geometry' stamp: 'mt 10/20/2010 14:58:44.824'!
useOptimalSize
	"Change the size of the view that only one kind/direction of scrolling is needed."

	self listOrientation = #default ifTrue: [
		self height: self viewport optimalSize y + 8.
		self width: self viewport optimalSize x + self scrollBarThickness +
			(self hasVerticalHeader ifTrue: [verticalHeader width] ifFalse: [0])].
		
	self listOrientation = #vertical ifTrue: [
		self height: self viewport optimalSize y + 8.
		self width: self viewport optimalSize x + self scrollBarThickness +
			(self hasVerticalHeader ifTrue: [verticalHeader width] ifFalse: [0])].

	self listOrientation = #horizontal ifTrue: [
		self width: self viewport optimalSize x + 8.
		self height: self viewport optimalSize y + self scrollBarThickness +
			(self hasHorizontalHeader ifTrue: [horizontalHeader width] ifFalse: [0])].! !

UiListIconView subclass: #UiListWidget
	instanceVariableNames: 'currentRow items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListWidget commentStamp: 'mt 10/19/2010 19:32:52.216' prior: 0!
Simple list. Item based. No model required.!

!UiListWidget class methodsFor: 'instance creation' stamp: 'mt 10/19/2010 19:38:28.314'!
new

	^ super new
		model: UiListWidgetModel new! !

!UiListWidget methodsFor: 'adding' stamp: 'mt 10/19/2010 20:03:04.117'!
addIcon: anIcon text: aLabel

	self model addItem: (UiListWidgetItem new text: aLabel; icon: anIcon).! !

!UiListWidget methodsFor: 'adding' stamp: 'mt 10/19/2010 20:02:59.691'!
addText: aLabel

	self model addItem: (UiListWidgetItem new text: aLabel).! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/19/2010 19:41:33.614'!
clear

	self model: UiListWidgetModel new.! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:15:08.334'!
currentIcon

	^ self currentItem ifNotNil: [:i | i icon]! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/20/2010 10:30:03.07'!
currentItem

	^ self currentRow ifNotNil: [:cc | self items at: cc]! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:14:43.107'!
currentText

	^ self currentItem ifNotNil: [:i | i text]! !

!UiListWidget methodsFor: 'initialization' stamp: 'mt 10/20/2010 13:50:29.946'!
initialize
	
	super initialize.
	self connect: self viewport signal: #activated: toSelector: #onActivated:.! !

!UiListWidget methodsFor: 'signals' stamp: 'mt 10/20/2010 10:35:08.772'!
itemActivated: aListWidgetItem

	self emit.! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/19/2010 19:40:25.443'!
items

	^ self model items! !

!UiListWidget methodsFor: 'updating' stamp: 'mt 10/20/2010 14:40:03.899'!
onActivated: aModelIndex

	self itemActivated: (aModelIndex
		ifNotNil: [self items at: aModelIndex row]).! !

!UiListWidget methodsFor: 'updating' stamp: 'mt 10/20/2010 14:46:04.736'!
onCurrentIndexChanged: aModelIndex

	super onCurrentIndexChanged: aModelIndex.
	self currentTextChanged: (aModelIndex ifNotNil: [:m | (self items at: m row) text]).! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:11:43.237'!
size

	^ self items size! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/19/2010 19:52:28.016'!
headerItem: aMorph
		
	headerItem := aMorph.
	self formatHeaderItem.! !

!UiViewSingleSelection methodsFor: 'add/remove' stamp: 'mt 10/20/2010 14:42:59.222'!
add: aCellIndex event: aUserInputEvent

	selectedCell := aCellIndex.
	self
		selectionChanged;
		currentCellChanged.! !

Object subclass: #UiListWidgetItem
	instanceVariableNames: 'icon text enabled model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiListWidgetItem commentStamp: 'mt 6/28/2010 17:06:54.526' prior: 0!
Container for item information to be used by UiListWidget.!

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:05:01.021'!
enabled: aBoolean

	enabled := aBoolean.
	self triggerItemChanged.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:05:09.972'!
icon: anObject

	icon := anObject.
	self triggerItemChanged.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:02:38.97'!
model

	^ model! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:02:44.327'!
model: aModel

	model := aModel.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:04:56.723'!
text: anObject

	text := anObject.
	self triggerItemChanged.! !

!UiListWidgetItem methodsFor: 'signal triggering' stamp: 'mt 10/19/2010 20:04:48.802'!
triggerItemChanged

	self model ifNotNil: [:m |
		m triggerItemChangedAt: (m items indexOf: self)].! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 10/19/2010 19:53:29.205'!
optimalSize
	"Return the height of the largest item."
	
	| max |
	max := 0.
	items do: [:item |
		item vResizing ~= #spaceFill
			ifTrue: [max := max max: item height]].
	self headerItem submorphs ifNotEmpty: [
		max := max max: self headerItem submorphBounds height].
	^ max! !

!UiListIconViewport methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:33:09.036'!
maxItemExtent

	^ maxItemExtent! !

!UiListIconViewport methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:53:11.793'!
optimalSize

	columns ifEmpty: [^ 1@1].

	self orientation = #default ifTrue: [
		^ columns first optimalSize @ self height].
	
	self orientation = #vertical ifTrue: [
		^ maxItemExtent x @ self height].
	
	self orientation = #horizontal ifTrue: [
		^ self width @ maxItemExtent y].! !

!UiListIconViewport methodsFor: 'geometry' stamp: 'mt 10/20/2010 14:53:43.833'!
updateExtent

	columns ifEmpty: [^ self].
	
	self orientation = #default ifTrue: [
		| newHeight optimalWidth |	
		newHeight := rows
			ifEmpty: [0]
			ifNotEmpty: [(rows collect: [:row | row height]) sum].
	
		optimalWidth := columns first optimalSize.
	
		^ optimalWidth < self owner width
			ifTrue: [self extent: self owner width - 5 @ newHeight]
			ifFalse: [self extent: optimalWidth @ newHeight]].
		
	self extent: (((self submorphBounds extent / maxItemExtent) ceiling * maxItemExtent)
		max: (self owner extent - 5)).! !

!UiListModel methodsFor: 'model data' stamp: 'mt 10/19/2010 19:53:37.273'!
textFor: aModelIndex

	^ self textAt: aModelIndex row! !

UiListModel subclass: #UiListWidgetModel
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiListWidgetModel commentStamp: 'mt 10/20/2010 14:12:24.921' prior: 0!
I am the hidden model used in the item-based UiListWidget.!

!UiListWidgetModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 20:02:09.394'!
addItem: aListWidgetItem

	self items add: aListWidgetItem.
	aListWidgetItem model: self.
	
	self triggerItemAddedAt: (self items indexOf: aListWidgetItem).! !

!UiListWidgetModel methodsFor: 'model data' stamp: 'mt 10/19/2010 19:38:05.057'!
iconAt: integer

	^ (self items at: integer) icon! !

!UiListWidgetModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 19:37:20.66'!
items

	^ items ifNil: [items := OrderedCollection new]! !

!UiListWidgetModel methodsFor: 'counts' stamp: 'mt 10/19/2010 19:37:11.329'!
rowCount

	^ self items size! !

!UiListWidgetModel methodsFor: 'model data' stamp: 'mt 10/19/2010 19:37:49.039'!
textAt: integer

	^ (self items at: integer) text! !

!UiListWidgetModel methodsFor: 'signal triggering' stamp: 'mt 10/19/2010 19:55:58.253'!
triggerItemAddedAt: index

	self rowsInsertedFor: nil from: index to: index.! !

!UiListWidgetModel methodsFor: 'signal triggering' stamp: 'mt 10/19/2010 19:56:13.227'!
triggerItemChangedAt: index

	self rowsChangedFor: nil from: index to: index.! !

!UiListWidgetModel methodsFor: 'signal triggering' stamp: 'mt 10/19/2010 19:56:25.038'!
triggerItemRemovedAt: index

	self rowsRemovedFor: nil from: index to: index.! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/19/2010 19:53:10.558'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | 
		item hResizing ~= #spaceFill
			ifTrue: [max := max max: (item width + (item valueOfProperty: #indent))]].
	self headerItem submorphs ifNotEmpty: [
		max := max max: self headerItem submorphBounds width + 5. "font rendering bugfix" ].
	^ max! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:19:31.999'!
clear

	listWidget clear.
	self updateCurrentRow: nil.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:55:55.354'!
color: aColor

	super color: aColor.
	readOnlyButton color: aColor.
	button color: aColor.
	listWidget borderColor: aColor muchDarker.! !

!UiComboBox methodsFor: 'finding' stamp: 'mt 10/20/2010 14:57:44.382'!
findText: aText

	listWidget items withIndexDo: [:item :index | item text = aText ifTrue: [^ index]].
	^ 0! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 10/20/2010 14:56:49.445'!
initialize

	super initialize.
	
	self
		vResizing: #shrinkWrap;
		changeTableLayout;
		listDirection: #leftToRight;
		cellPositioning: #topLeft;
		width: 150.
		
	button := UiToolButton new
		text: nil;
		checkable: true;
		vResizing: #rigid;
		icon: (UiSilkIcons bulletArrowDownIcon clippedTo: 8@8);
		layoutInset: (3@4 corner: 3@4).
	lineEdit := UiLineEdit new
		outerBorderWidth: 0.
	listWidget := UiListWidget new
		borderWidth: 1.
	
	readOnlyButton := UiToolButton new
		text: ' ';
		icon: nil;
		checkable: true;
		hResizing: #spaceFill.
	
	"Initialize the submorphs the first time."
	self readOnly: true.
		
	self connect: listWidget signal: #currentRowChanged: toSelector: #updateCurrentRow:.
	self connect: listWidget signal: #currentRowChanged: toSelector: #hideList.

	self connect: button signal: #toggled: toSelector: #listVisible:.
	self connect: button signal: #pressed toSelector: #showList.

	self connect: readOnlyButton signal: #pressed toSelector: #showList.
	self connect: readOnlyButton signal: #toggled: to: button selector: #checked:.

	self connect: button signal: #toggled: to: readOnlyButton selector: #checked:.
	
	self color: (Color gray: 0.8).! !

!UiComboBox methodsFor: 'private' stamp: 'mt 10/20/2010 14:50:47.379'!
showList

	listWidget
		useOptimalSize;
		width: (self width + 2 max: listWidget width);
		height: (listWidget height min: 100);
		topLeft: (self localPointToGlobal: self bottomLeft) - (1@0);
		openInWorld.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 10/20/2010 14:11:32.602'!
size

	^ listWidget size! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 10/20/2010 14:15:18.676'!
updateCurrentRow: anIndex

	lineEdit theText: listWidget currentText.
	
	readOnlyButton text: lineEdit theText.
	readOnlyButton icon: listWidget currentIcon.
	
	self currentIndexChanged: anIndex text: (self converter value: lineEdit theText).! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/20/2010 13:54:10.953'!
currentIndexChanged: aModelIndex

	self emit.! !

!UiItemViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 10/20/2010 14:05:41.933'!
currentModelIndex

	^ self modelIndexFor: self selection currentCell! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/20/2010 14:06:02.419'!
onCurrentCellChanged

	self currentIndexChanged: self currentModelIndex.! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/20/2010 14:07:02.107'!
onSelectionChanged

	self selectionChanged: self selectedModelIndices.! !

!UiItemViewport methodsFor: 'rows/columns - to model index' stamp: 'mt 10/20/2010 14:07:17.985'!
selectedModelIndices

	^ self selection cells collect: [:cellIndex |
		self modelIndexFor: cellIndex]! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/20/2010 14:43:53.154'!
selection: aSelection

	selection := aSelection.
	
	self connect: aSelection signal: #selectionChanged toSelector: #redrawLater.
	self connect: aSelection signal: #selectionChanged toSelector: #updateFocus.
	self connect: aSelection signal: #currentCellChanged toSelector: #redrawLater.
	self connect: aSelection signal: #selectionChanged toSelector: #onSelectionChanged.
	self connect: aSelection signal: #currentCellChanged toSelector: #onCurrentCellChanged.
	
	self redrawLater.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/20/2010 13:45:49.758'!
selectionChanged: someModelIndices

	self emit.! !
