
!UiAbstractItemView methodsFor: 'initialization' stamp: 'mt 10/14/2010 20:26:47.648'!
initialize

	"Initialize headers."
	verticalHeader := Morph new color: Color transparent; width: self scrollBarThickness; changeTableLayout; listDirection: #topToBottom.
	horizontalHeader := Morph new color: Color transparent; height: self scrollBarThickness; changeTableLayout; listDirection: #leftToRight.
	vHeaderScroller := TransformMorph new color: Color transparent.
	hHeaderScroller := TransformMorph new color: Color transparent.

	super initialize.
	
	self
		borderWidth: 0;
		color: Color white.
	
	"Initialize viewport."
	self scroller addMorph: self viewportClass new.
	self connect: self viewport signal: #focusChanged: toSelector: #scrollToShow:.
	self connect: self viewport signal: #clicked: toSelector: #clicked:.
	self connect: self viewport signal: #doubleClicked: toSelector: #doubleClicked:.
	self viewport verticalHeader: verticalHeader.
	self viewport horizontalHeader: horizontalHeader.
	
	"Initialize item delegates."
	itemDelegates := Dictionary new.
	self defaultItemDelegate: UiSimpleItemDelegate new.
	
	"Initialize headers - Finish."
	verticalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #height: pattern: #(2).
	vHeaderScroller addMorph: verticalHeader.

	horizontalHeader connect: self viewport signal: #resizedToWidth:height: toSelector: #width:.
	hHeaderScroller addMorph: horizontalHeader.! !

!UiAbstractItemView methodsFor: 'selection' stamp: 'mt 10/19/2010 10:25:19.838'!
selection
	"Use this selection object to be notified whenever a selection changes."
	
	^ self viewport selection! !

UiAbstractItemView subclass: #UiListIconView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Views'!

!UiListIconView commentStamp: 'mt 10/14/2010 15:39:41.542' prior: 0!
I am a list view that is able to break the line of items in one direction and shows as many items as possible.!

!UiListIconView methodsFor: 'initialization' stamp: 'mt 10/14/2010 17:27:39.015'!
initialize

	super initialize.
	
	self defaultItemDelegate: UiSimpleItemDelegate new.! !

!UiViewInfo methodsFor: 'accessing' stamp: 'mt 10/14/2010 13:34:19.717'!
headerItem: aMorph
		
	headerItem := aMorph.
	self formatHeaderItem.! !

!UiViewSingleSelection methodsFor: 'add/remove' stamp: 'mt 10/11/2010 09:21:51.677'!
add: aCellIndex event: aUserInputEvent

	selectedCell := aCellIndex.
	self selectionChanged.! !

UiListView subclass: #UiListWidget
	instanceVariableNames: 'currentRow items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Core'!

!UiListWidget commentStamp: 'mt 6/28/2010 17:01:05.359' prior: 0!
Simple list. No model required.!

!UiListWidget class methodsFor: 'instance creation' stamp: 'mt 6/29/2010 09:56:04.041'!
new

	| instance |
	instance := super new.
	
	(instance
		on: instance
		list: #labels
		selected: #currentRow
		changeSelected: #currentRow:
		menu: nil
		keystroke: #arrowKey:from:)
		getIconsSelector: #icons.
		
	^ instance! !

!UiListWidget methodsFor: 'adding' stamp: 'mt 6/29/2010 09:50:21.913'!
addIcon: anIcon text: aLabel

	self addItem: (UiListWidgetItem new text: aLabel; icon: anIcon).! !

!UiListWidget methodsFor: 'adding' stamp: 'mt 6/29/2010 16:50:08.325'!
addItem: aListWidgetItem

	aListWidgetItem listWidget: self.
	self items add: aListWidgetItem.
	
	self changed: #labels.! !

!UiListWidget methodsFor: 'adding' stamp: 'mt 6/29/2010 09:50:03.894'!
addText: aLabel

	self addItem: (UiListWidgetItem new text: aLabel).! !

!UiListWidget methodsFor: 'as yet unclassified' stamp: 'mt 7/21/2010 18:18:01.159'!
clear

	items := nil.
	self changed: #icons; changed: #labels.! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 9/8/2010 12:00:52.377'!
currentItem

	self currentRow = 0 ifTrue: [^ nil].
	^ self items at: self currentRow! !

!UiListWidget methodsFor: 'signals' stamp: 'mt 6/29/2010 09:53:26.556'!
currentItemChanged: currentItem previous: previousItem

	self emit.! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 6/28/2010 17:05:50.882'!
currentRow

	^ currentRow ifNil: [currentRow := 0]! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 6/29/2010 17:04:55.544'!
currentRow: anIndex

	| prevIndex |
	anIndex == currentRow ifTrue: [^ self].
	
	prevIndex := currentRow.
	
	currentRow := anIndex.
	self changed: #currentRow.
	
	self currentRowChanged: currentRow.
	
	self currentItemChanged: (self items at: currentRow ifAbsent: [nil]) previous: (self items at: prevIndex ifAbsent: [nil]).
	self currentTextChanged: ((self items at: currentRow ifAbsent: [nil]) ifNotNil: [:i | i text]).! !

!UiListWidget methodsFor: 'signals' stamp: 'mt 6/29/2010 09:52:55.78'!
currentRowChanged: currentRow

	self emit.! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 6/29/2010 09:46:34.46'!
icons

	^ self items collect: [:item | item icon]! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 6/28/2010 17:09:17.87'!
items

	^ items ifNil: [items := OrderedCollection new].! !

!UiListWidget methodsFor: 'accessing' stamp: 'mt 6/29/2010 09:55:53.211'!
labels

	^ self items collect: [:item | item text]! !

Object subclass: #UiListWidgetItem
	instanceVariableNames: 'listWidget icon text enabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Widgets-Support'!

!UiListWidgetItem commentStamp: 'mt 6/28/2010 17:06:54.526' prior: 0!
Container for item information to be used by UiListWidget.!

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 6/28/2010 21:22:23.451'!
enabled: aBoolean

	enabled := aBoolean.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 7/21/2010 18:14:02.048'!
icon: anObject

	icon := anObject.
	self listWidget changed: #icons.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 6/29/2010 09:47:12.376'!
listWidget

	^ listWidget! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 7/21/2010 17:55:42.438'!
listWidget: aListWidget

	listWidget := aListWidget.! !

!UiListWidgetItem methodsFor: 'accessing' stamp: 'mt 7/21/2010 17:54:54.418'!
text: anObject

	text := anObject.
	self listWidget changed: #labels.! !

!UiViewRowInfo methodsFor: 'geometry' stamp: 'mt 10/11/2010 16:37:58.44'!
optimalSize
	"Return the height of the largest item."
	
	| max |
	max := 0.
	items do: [:item |
		item vResizing ~= #spaceFill
			ifTrue: [max := max max: item height]].
	max := max max: self headerItem submorphBounds height.
	^ max! !

!UiListIconViewport methodsFor: 'geometry' stamp: 'mt 10/14/2010 17:24:47.645'!
updateExtent

	self orientation = #default ifTrue: [
		| newHeight optimalWidth |	
		newHeight := rows
			ifEmpty: [0]
			ifNotEmpty: [(rows collect: [:row | row height]) sum].
	
		optimalWidth := columns first optimalSize.
	
		^ optimalWidth < self owner width
			ifTrue: [self extent: self owner width - 5 @ newHeight]
			ifFalse: [self extent: optimalWidth @ newHeight]].
		
	self extent: (((self submorphBounds extent / maxItemExtent) ceiling * maxItemExtent)
		max: (self owner extent - 5)).! !

!UiListModel methodsFor: 'model data' stamp: 'mt 10/14/2010 19:46:33.425'!
textFor: aModelIndex

	^ self textAt: aModelIndex row! !

!UiViewColumnInfo methodsFor: 'geometry' stamp: 'mt 10/14/2010 20:17:06.246'!
optimalSize
	"Return the width of the largest item."
	
	| max |
	max := 0.
	items do: [:item | 
		item hResizing ~= #spaceFill
			ifTrue: [max := max max: (item width + (item valueOfProperty: #indent))]].
	max := max max: self headerItem submorphBounds width + 5. "font rendering bugfix"
	^ max! !

!UiComboBox methodsFor: 'as yet unclassified' stamp: 'mt 7/21/2010 19:27:57.849'!
clear

	listWidget clear.
	self updateCurrentRow: 0.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 10/19/2010 19:15:45.296'!
color: aColor

	super color: aColor.
	readOnlyButton color: aColor.
	button color: aColor.! !

!UiComboBox methodsFor: 'finding' stamp: 'mt 6/30/2010 08:22:08.58'!
findText: aText

	listWidget labels withIndexDo: [:label :index | label = aText ifTrue: [^ index]].
	^ 0! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 10/19/2010 19:25:29.356'!
initialize

	super initialize.
	
	self
		vResizing: #shrinkWrap;
		changeTableLayout;
		listDirection: #leftToRight;
		cellPositioning: #topLeft;
		width: 150.
		
	super color: Color transparent.
		
	button := UiToolButton new
		text: nil;
		checkable: true;
		vResizing: #rigid;
		icon: (UiSilkIcons bulletArrowDownIcon clippedTo: 8@8);
		layoutInset: (3@4 corner: 3@4).
	lineEdit := UiLineEdit new
		outerBorderWidth: 0.
	listWidget := UiListWidget new autoDeselect: false; vResizing: #shrinkWrap.
	
	readOnlyButton := UiToolButton new
		checkable: true;
		hResizing: #spaceFill.
	
	"Initialize the submorphs the first time."
	self readOnly: true.
		
	self connect: listWidget signal: #currentRowChanged: toSelector: #updateCurrentRow:.
	self connect: listWidget signal: #currentRowChanged: toSelector: #hideList.
	self connect: button signal: #toggled: toSelector: #listVisible:.
	self connect: button signal: #pressed toSelector: #showList.
	self connect: readOnlyButton signal: #pressed toSelector: #showList.
	
	self connect: readOnlyButton signal: #toggled: to: button selector: #checked:.
	self connect: button signal: #toggled: to: readOnlyButton selector: #checked:.
! !

!UiComboBox methodsFor: 'private' stamp: 'mt 10/19/2010 19:27:37.864'!
showList

	listWidget
		updateListExtent;
		width: (self width + 2 max: listWidget width);
		topLeft: (self localPointToGlobal: self bottomLeft) - (1@0);
		openInWorld.! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 9/20/2010 13:59:43.751'!
updateCurrentRow: anIndex

	lineEdit theText: (listWidget labels at: anIndex ifAbsent: [nil]).
	
	readOnlyButton text: lineEdit theText.
	readOnlyButton icon: (listWidget icons at: anIndex ifAbsent: [nil]).
	
	self currentIndexChanged: anIndex text: (self converter value: lineEdit theText).! !

!UiItemViewport methodsFor: 'selection' stamp: 'mt 10/19/2010 10:19:23.649'!
selection: aSelection

	selection := aSelection.
	
	self connect: aSelection signal: #selectionChanged toSelector: #redrawLater.
	self connect: aSelection signal: #selectionChanged toSelector: #updateFocus.
	self connect: aSelection signal: #currentCellChanged toSelector: #redrawLater.
	
	self redrawLater.! !

!UiItemViewport methodsFor: 'signals' stamp: 'mt 10/8/2010 16:07:34.586'!
selectionChanged: aModelIndex

	self emit.! !
